/*****************************************
* irc.c - IRC related subroutines
*
* CVS $Id: irc.c,v 1.27 2000/07/06 16:39:29 wcampbel Exp $
*****************************************/

#include "split.h"

void Register()
{
	int i;
#ifdef IRCD_HYBRID
        sts("PASS %s :TS", me.pass);
        sts("SERVER %s 1 :%s", me.name, me.desc);
#endif
#ifdef IRCD_DAL
        sts("PASS %s", me.pass);
        sts("SERVER %s 1 :%s", me.name, me.desc);
#endif
	notice = time(NULL) + me.delay;
	is_delayed = 1;
	for (i=0; i<MAXSERVER; i++) {
		server_table[i] = NULL;
	}
	map_head = NewServer(me.name, NULL);
	for (i=0; i<MAXSERVER; i++) {
		split_table[i] = NULL;
	}
}

void InitSplitServ()
{
#ifdef IRCD_HYBRID
	sts("NICK %s 1 1 + %s %s %s :%s", SplitServ.nick,
                SplitServ.user, SplitServ.host, me.name, me.desc);
	sts(":%s SJOIN %ld %s + :%s", me.name, time(NULL), me.join_chan,
		SplitServ.nick);
#endif
#ifdef IRCD_DAL
        sts("NICK %s 1 1 %s %s %s 0 :%s", SplitServ.nick,
                SplitServ.user, SplitServ.host, me.name, me.desc);
	sts(":%s JOIN %s", SplitServ.nick, me.join_chan);
#endif
}

void Parse(char * buf)
{
        char *origin, *cmd, *coreLine;
        int cmdptr = 0;

        strip(buf);

        if (!(*buf))
                return;

        coreLine = strdup(buf);

#ifdef DEBUG
        log("R: %s", buf);
#endif
        origin = strtok(buf, " ");
        if (*origin == ':')
                cmdptr = 1;
        cmd = strtok(NULL, " ");

        if (cmdptr) {
                origin++;
		if (!strcmp(cmd, "SERVER")) {
			char *o, *c;
			cmd = strtok(NULL, " ");
			o = strdup(origin);
			c = strdup(cmd);
			if (!is_delayed) {
				log("New Server: %s to %s", c, o);
				sts(":%s PRIVMSG %s :CONNECT: %s to %s", 
					SplitServ.nick, me.join_chan, c,
					o);
			}
			NewServer(c, o);
			DelSplit(c);
		} else if (!strcmp(cmd, "PING")) {
			char * cmd = strtok(NULL, " ");
			char * p = strtok(NULL, " ");
			p++;
			sts(":%s PONG %s :%s", me.name, p, cmd);
		} else if (!strcmp(cmd, "PRIVMSG")) {
			char * target = strtok(NULL, " ");
			char * cmd = strtok(NULL, " ");
			char * p = strtok(NULL, " ");
			cmd++;
			if (target[0] == '#') {
				if (cmd[0] == '!') {
					if (!strcmp(cmd, "!split")) {
						PrintSplit(target);
					} else if (!strcmp(cmd, "!map")) {
						PrintMap(target, map_head, 0);
					} else if (!strcmp(cmd, "!dump")) {
						PrintDump(target);
					}
				}
			} else {
					
				if (!strcmp(cmd, "!split")) {
					PrintSplit(origin);
				} else if (!strcmp(cmd, "!map")) {
					PrintMap(origin, map_head, 0);
				} else if (!strcmp(cmd, "!dump")) {
					PrintDump(origin);
				} else if (!strcmp(cmd, "die")) {
					Shutdown(origin, p);
				} else if (!strcmp(cmd, "restart")) {
					Restart(origin, p);
				} else if (!strcmp(cmd, "shutdown")) {
					Shutdown(origin, p);
				} else if (!strcmp(cmd, "help")) {
					PrintHelp(origin, p);
				}
			}
		}
	} else {
		if (!strcmp(origin, "PING")) {
                        sts("PONG %s", cmd);
		} else if (!strcmp(origin, "PASS")) {
			InitSplitServ();
		} else if (!strcmp(origin, "SERVER")) {
			char *c = strdup(cmd);
			if (!is_delayed) {
				log("New Server: %s", c);
				sts(":%s PRIVMSG %s :CONNECT: %s to %s", 
					SplitServ.nick, me.join_chan, c,
					me.name);
			}
			NewServer(c, me.name);
			DelSplit(c);
		} else if (!strcmp(origin, "SQUIT")) {
			char * serv = cmd;
			char * s = strdup(serv);
			cmd = strtok(NULL, "\n");
			cmd++;
			log("Split: %s - %s", s, cmd);
			if (!is_delayed) {
				sts(":%s PRIVMSG %s :SPLIT: %s - %s",
					SplitServ.nick, me.join_chan, 
					serv, cmd);
			}
			DelServer(s);
			NewSplit(s);
		}
	}

	free(coreLine);
}

void PrintSplit(char* target)
{
	int i;
	int count = 0;
	for (i=0; i<MAXSERVER; i++) {
		if (split_table[i]) {
			count++;
			sts(":%s PRIVMSG %s :SPLIT SERVER: %s",
				SplitServ.nick, target, split_table[i]->name);
		}
	}
	if (!count) {
		sts(":%s PRIVMSG %s :NO SPLITS", SplitServ.nick, target);
	}
}

void PrintMap(char* target, Server* s, int level)
{
	int i;
	char* d = dash(level);
	if (s) {
		sts(":%s PRIVMSG %s :%s %s", SplitServ.nick, target,
	    		d, s->name);
		for (i=0; i<MAXSERVER; i++) {
			if (s->leaflist[i]) {
				if (s->leaflist[i]->name) {
					PrintMap(target, s->leaflist[i],
						level+1);
				}
			}
		}
	}
	free(d);
}

char* dash(int level)
{
	int i;
	char* s = malloc(MAXDEPTH*3);
	s[0] = '\0';
	if (!level) {
		return s;
	}
	for (i=level; i>=0; i--) {
		if (!i) {
			sprintf(s, "%s`-", s);
		} else {
			sprintf(s, "%s    ", s);
		}
	}
	return s;
}

void PrintDump(char * target)
{
	int i,j;
	char leaves[BUFSIZE] = "";
	for (i=0; i<MAXSERVER; i++) {
		leaves[0] = '\0';
		if (server_table[i]) {
			for (j=0; j<MAXSERVER; j++) {
				if (server_table[i]->leaflist[j]) {
					sprintf(leaves, "%s %s", leaves,
					  server_table[i]->leaflist[j]->name);
				}
			}
			sts(":%s PRIVMSG %s :Server: %s Leaves:%s", 
				SplitServ.nick,target, server_table[i]->name, 
				(!strcmp(leaves, "")) ? " None" : leaves);
		}
	}
}

void Shutdown(char* target, char* pass)
{
    if (pass) {
	if (!strcmp(pass, me.adpass)) {
#ifdef IRCD_HYBRID
		sts(":%s WALLOPS :Shutdown by %s", me.name, target);
#endif
#ifdef IRCD_DAL
		sts(":%s GLOBOPS :Shutdown by %s", me.name, target);
#endif
		sts(":%s QUIT :Shutdown by %s", SplitServ.nick, target);
		close(servsock);
		exit(0);
	}
    }
    sts(":%s PRIVMSG %s :Incorrect Password", SplitServ.nick, target);
}

void Restart(char * target, char * pass)
{
    if (pass) {
	if (!strcmp(pass, me.adpass)) {
#ifdef IRCD_HYBRID
                sts(":%s WALLOPS :Restart by %s", SplitServ.nick, target);
#endif
#ifdef IRCD_DAL
                sts(":%s GLOBOPS :Restart by %s", SplitServ.nick, target);
#endif
                sts(":%s QUIT :Restart by %s", SplitServ.nick, target);
                close(servsock);
		execv(SPATH, NULL);
	}
    }
    sts(":%s PRIVMSG %s :Incorrect Password", SplitServ.nick, target);
}

void PrintHelp(char * target, char * cmd)
{
	if (cmd) {
		if (!strcmp(cmd, "!map")) {
			sts(":%s PRIVMSG %s :Prints the server map", 
				SplitServ.nick, target);
		} else if (!strcmp(cmd, "!split")) {
			sts(":%s PRIVMSG %s :Prints the list of split servers",
				SplitServ.nick, target);
		} else if (!strcmp(cmd, "!dump")) {
			sts(":%s PRIVMSG %s :Prints all connected servers and their leaves",
				SplitServ.nick, target);
		} else if (!strcmp(cmd, "shutdown")) {
			sts(":%s PRIVMSG %s :Shuts down the service (syntax: shutdown <password>)",
				SplitServ.nick, target);
		} else if (!strcmp(cmd, "die")) {
			sts(":%s PRIVMSG %s :Shuts down the service (syntax: die <password>)",
				SplitServ.nick, target);
		} else if (!strcmp(cmd, "restart")) {
			sts(":%s PRIVMSG %s :Restarts the service (syntax: restart <password>)",
				SplitServ.nick, target);
		}
	} else {
		sts(":%s PRIVMSG %s :Commands:  !map, !split, !dump, shutdown, die, restart, help",
		SplitServ.nick, target);
	}
}
